import os
import re

from cliff.command import Command
import progressbar

from . import common
from .config import get_final_config


class Download(Command):
    'Download data files from a specific competition.'

    def get_parser(self, prog_name):
        parser = super(Download, self).get_parser(prog_name)

        parser.add_argument('-c', '--competition', help='competition')
        parser.add_argument('-u', '--username', help='username')
        parser.add_argument('-p', '--password', help='password')
        parser.add_argument('-f', '--filename', help='filename')

        return parser

    def take_action(self, parsed_args):
        config = get_final_config(parsed_args)

        username = config.get('username', '')
        password = config.get('password', '')
        competition = config.get('competition', '')
        file_name = parsed_args.filename

        browser = common.login(username, password)
        base = 'https://www.kaggle.com'
        data_url = '/'.join([base, 'c', competition, 'data'])

        data_page = browser.get(data_url)

        if data_page.status_code == 404:
            print('competition not found')
            return

        data = str(data_page.soup)
        links = re.findall(
            '"url":"(/c/{}/download/[^"]+)"'.format(competition), data
        )

        if not links:  # fallback for inclass competition
            links = map(
                lambda link: link.get('href'),
                data_page.soup.find(id='data-files').find_all('a')
            )

        if not links:
            print('not found')

        for link in links:
            url = base + link
            if file_name is None or url.endswith('/' + file_name):
                if self.download_file(browser, url) is False:
                    return

    def download_file(self, browser, url):
        print('downloading {}\n'.format(url))
        local_filename = url.split('/')[-1]
        headers = {}
        done = False
        file_size = 0
        content_length = int(
            browser.request('head', url).headers.get('Content-Length')
        )

        bar = progressbar.ProgressBar()
        widgets = [local_filename, ' ', progressbar.Percentage(), ' ',
                   progressbar.Bar(marker='#'), ' ',
                   progressbar.ETA(), ' ', progressbar.FileTransferSpeed()]

        if os.path.isfile(local_filename):
            file_size = os.path.getsize(local_filename)
            if file_size < content_length:
                headers['Range'] = 'bytes={}-'.format(file_size)
            else:
                done = True

        finished_bytes = file_size

        if file_size == content_length:
            print('{} already downloaded !'.format(local_filename))
            return
        elif file_size > content_length:
            print('Something wrong here, Incorrect file !')
            return
        else:
            bar = progressbar.ProgressBar(widgets=widgets,
                                          maxval=content_length).start()
            bar.update(finished_bytes)

        if not done:
            stream = browser.get(url, stream=True, headers=headers)

            if not self.is_downloadable(stream):
                warning = (
                    'Warning: '
                    'download url for file {} resolves to an html document '
                    'rather than a downloadable file. \n'
                    'Is it possible you have not '
                    'accepted the competition\'s rules on the kaggle website?'
                    .format(local_filename)
                )
                print('\n\n{}\n'.format(warning))
                return False

            with open(local_filename, 'ab') as f:
                for chunk in stream.iter_content(chunk_size=1024):
                    if chunk:  # filter out keep-alive new chunks
                        f.write(chunk)
                        finished_bytes += len(chunk)
                        bar.update(finished_bytes)

            bar.finish()
            print('')

        return True

    def is_downloadable(self, response):
        '''
        Checks whether the response object is a html page
        or a likely downloadable file.
        Intended to detect error pages or prompts
        such as kaggle's competition rules acceptance prompt.

        Returns True if the response is a html page. False otherwise.
        '''

        content_type = response.headers.get('Content-Type', '')
        content_disp = response.headers.get('Content-Disposition', '')

        if 'text/html' in content_type and 'attachment' not in content_disp:
            # This response is a html file
            # which is not marked as an attachment,
            # so we likely hit a rules acceptance prompt
            return False
        return True


class Dataset(Download):
    'Download dataset from a specific user.'

    def get_parser(self, prog_name):
        parser = super(Dataset, self).get_parser(prog_name)
        parser.add_argument('-d', '--dataset', help='dataset')
        parser.add_argument('-o', '--owner', help='owner')
        return parser

    def take_action(self, parsed_args):
        config = get_final_config(parsed_args)

        username = config.get('username', '')
        password = config.get('password', '')
        dataset = parsed_args.dataset
        owner = parsed_args.owner
        file_name = parsed_args.filename

        browser = common.login(username, password)
        base = 'https://www.kaggle.com'
        data_url = '/'.join([base, owner, dataset])

        data_page = browser.get(data_url)

        if data_page.status_code == 404:
            print('dataset not found')
            return

        data = str(data_page.soup)
        links = re.findall(
            '"url":"(/{}/{}/downloads/[^"]+)"'.format(owner, dataset), data
        )

        for link in links:
            url = base + link
            if file_name is None or url.endswith('/' + file_name):
                if self.download_file(browser, url) is False:
                    return
